'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

exports.generateInputComponent = generateInputComponent;
exports.default = generateInputField;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactRedux = require('react-redux');

var _redux = require('redux');

var _lodash = require('lodash');

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _withFormId = require('./withFormId');

var _withFormId2 = _interopRequireDefault(_withFormId);

var _actions = require('../actions');

var _selectors = require('../selectors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Generate an input field with common input functionality.
 *
 * @param  {string} type      This should be a valid input[type] string.
 * @param  {object} options   Validations that should be run on the input
 *   field by default e.g. email type fields should run an email validation without
 *   the developer having to explicitly specify that every time they want to use
 *   the component.
 * @return {Component}        A `InputField` component that renders the input
 *   according to the specified options.
 */
function generateInputComponent(type) {
  var _class, _temp;

  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var _ref$validate = _ref.validate;
  var validate = _ref$validate === undefined ? '' : _ref$validate;
  var className = _ref.className;

  return _temp = _class = function (_React$Component) {
    (0, _inherits3.default)(InputField, _React$Component);

    function InputField(props, context) {
      (0, _classCallCheck3.default)(this, InputField);

      var _this = (0, _possibleConstructorReturn3.default)(this, (InputField.__proto__ || (0, _getPrototypeOf2.default)(InputField)).call(this, props, context));

      (0, _invariant2.default)(_this.context.form, 'Input type elements must be contained within a valid `Form` component!');

      _this.onChange = _this.onChange.bind(_this);
      _this.onFocus = _this.onFocus.bind(_this);
      _this.onBlur = _this.onBlur.bind(_this);
      _this.triggerAsyncValidation = _this.triggerAsyncValidation.bind(_this);
      return _this;
    }

    (0, _createClass3.default)(InputField, [{
      key: 'componentWillMount',
      value: function componentWillMount() {
        if (this.props.name) {
          this.context.form.attach({
            name: this.props.name,
            initialValue: this.props.value,
            validationString: this.props.validate + '|' + validate,
            asyncValidators: this.props.validateAsync
          });
        }
      }
    }, {
      key: 'shouldComponentUpdate',
      value: function shouldComponentUpdate() {
        // TODO: Implement shouldComponentUpdate in generateInputField
        return true;
      }
    }, {
      key: 'componentWillUnmount',
      value: function componentWillUnmount() {
        this.context.form.detach({ name: this.props.name });
      }

      /**
       * Methods that dispatch actions.
       */

    }, {
      key: 'onChange',
      value: function onChange(e) {
        if (this.props.onChange && this.props.onChange(e) || !this.props.onChange) {
          this.props.change({
            id: this.props.formId,
            name: this.props.name,
            value: e.target.value
          });
        }
      }
    }, {
      key: 'onFocus',
      value: function onFocus() {
        this.props.focus({ id: this.props.formId, name: this.props.name });
        if (this.props.onFocus) this.props.onFocus();
      }
    }, {
      key: 'triggerAsyncValidation',
      value: function triggerAsyncValidation() {
        var _props = this.props;
        var validateAsync = _props.validateAsync;
        var formId = _props.formId;
        var name = _props.name;
        var value = _props.value;
        var dispatch = _props.dispatch;

        (0, _lodash.forEach)(validateAsync, function (validateActionRequest, validator) {
          var action = validateActionRequest(formId, name, validator, (0, _defineProperty3.default)({}, name, value));
          // The action may be noop, if the components decide there is no need to
          // validate
          if (action && action.type) dispatch(action);
        });
      }
    }, {
      key: 'shouldPerformAsyncValidation',
      value: function shouldPerformAsyncValidation() {
        return this.props.needsValidation && !this.props.hasSyncErrors;
      }
    }, {
      key: 'onBlur',
      value: function onBlur() {
        var _props2 = this.props;
        var formId = _props2.formId;
        var name = _props2.name;
        var onBlur = _props2.onBlur;

        this.props.blur({ id: formId, name: name });
        if (this.shouldPerformAsyncValidation()) this.triggerAsyncValidation();
        if (onBlur) onBlur();
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement('input', {
          type: type,
          name: this.props.name,
          id: this.props.name,
          value: this.props.value,
          className: (0, _classnames2.default)(className, this.props.className),
          autoFocus: this.props.autoFocus || false,
          disabled: this.props.disabled || false,
          placeholder: this.props.placeholder || '',
          required: this.props.required || false,
          onKeyUp: this.props.onKeyUp || function () {},
          onChange: this.onChange,
          onFocus: this.onFocus,
          onBlur: this.onBlur
        });
      }
    }]);
    return InputField;
  }(_react2.default.Component), _class.displayName = (0, _lodash.capitalize)((0, _lodash.camelCase)(type)) + 'Field', _class.propTypes = {
    // User defined options
    placeholder: _react2.default.PropTypes.string,
    className: _react2.default.PropTypes.string,
    name: _react2.default.PropTypes.string,
    autoFocus: _react2.default.PropTypes.bool,
    disabled: _react2.default.PropTypes.bool,
    required: _react2.default.PropTypes.bool,
    validate: _react2.default.PropTypes.string,
    validateAsync: _react2.default.PropTypes.object,
    formId: _react2.default.PropTypes.string.isRequired,
    // Event callbacks
    onKeyUp: _react2.default.PropTypes.func,
    onChange: _react2.default.PropTypes.func,
    onFocus: _react2.default.PropTypes.func,
    onBlur: _react2.default.PropTypes.func,
    // Connected values
    value: _react2.default.PropTypes.string.isRequired,
    touched: _react2.default.PropTypes.bool.isRequired,
    needsValidation: _react2.default.PropTypes.bool.isRequired,
    hasSyncErrors: _react2.default.PropTypes.bool.isRequired,
    // Dispatch methods
    blur: _react2.default.PropTypes.func,
    focus: _react2.default.PropTypes.func,
    change: _react2.default.PropTypes.func,
    dispatch: _react2.default.PropTypes.func
  }, _class.defaultProps = {
    validate: '',
    validateAsync: {}
  }, _class.contextTypes = {
    form: _react2.default.PropTypes.object.isRequired
  }, _temp;
}

/**
 * Generate a connected input field component that can function with form
 * components.
 */
function generateInputField() {
  var InputField = generateInputComponent.apply(undefined, arguments);

  var mapStateToProps = function mapStateToProps(state, _ref2) {
    var formId = _ref2.formId;
    var name = _ref2.name;
    return {
      value: (0, _selectors.getFieldValue)(formId, name)(state),
      touched: (0, _selectors.getFieldTouched)(formId, name)(state),
      needsValidation: (0, _selectors.getFieldNeedsValidation)(formId, name)(state),
      hasSyncErrors: (0, _selectors.getFieldHasSyncErrors)(formId, name)(state)
    };
  };
  var mapDispatchToProps = function mapDispatchToProps(dispatch) {
    return {
      change: (0, _redux.bindActionCreators)(_actions.change, dispatch),
      focus: (0, _redux.bindActionCreators)(_actions.focus, dispatch),
      blur: (0, _redux.bindActionCreators)(_actions.blur, dispatch),
      dispatch: dispatch
    };
  };
  return (0, _withFormId2.default)((0, _reactRedux.connect)(mapStateToProps, mapDispatchToProps)(InputField));
}