'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFormValidationAction = exports.createFormSubmitAction = undefined;

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Create an asynchronous sumission action that the Form component understands,
 * namely using promises to atain communication between different sagas.
 *
 * @param  {function} requestAction
 *         An action creator to trigger the request action.
 * @param  {array} types
 *         An array containing two constants, the first should be the success
 *         and the second the failure constants to the request.
 *
 * @return {function}
 *         Returns a function that accepts the form id, the form values and the
 *         dispatch method.
 *         This function then returns a promise which the Form saga can
 *         understand.
 */
var createFormSubmitAction = exports.createFormSubmitAction = function createFormSubmitAction(requestAction, types) {
  var _types = (0, _slicedToArray3.default)(types, 2);

  var successActionType = _types[0];
  var failureActionType = _types[1];


  return function (id, data) {
    var _ref = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

    var onSubmitSuccess = _ref.onSubmitSuccess;
    var onSubmitFailure = _ref.onSubmitFailure;
    return {
      type: _constants.SUBMIT_REQUEST,
      meta: { successActionType: successActionType, failureActionType: failureActionType, onSubmitSuccess: onSubmitSuccess, onSubmitFailure: onSubmitFailure },
      payload: {
        id: id,
        action: requestAction(data)
      }
    };
  };
};

/**
 * Create an asynchronous validation action that the Form component understands.
 *
 * @param  {function} requestAction
 *         An action creator to trigger the request action.
 * @param  {array} types
 *         An array containing two constants, the first should be the success
 *         and the second the failure constants to the request.

 * @return {function}
 *         Returns a function that accepts the form id, the form values and the
 *         dispatch method.
 *         This function then returns a promise which the Form saga can
 *         understand.
 */
var createFormValidationAction = exports.createFormValidationAction = function createFormValidationAction(requestAction, types) {
  var _types2 = (0, _slicedToArray3.default)(types, 2);

  var successActionType = _types2[0];
  var failureActionType = _types2[1];


  return function (id, name, validator, data) {
    return {
      type: _constants.VALIDATION_REQUEST,
      meta: { successActionType: successActionType, failureActionType: failureActionType },
      payload: {
        id: id,
        name: name,
        validator: validator,
        action: requestAction(data)
      }
    };
  };
};