'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.field = exports.Field = exports.fieldNeedsValidation = undefined;

var _immutable = require('immutable');

var _lodash = require('lodash');

var _validators = require('./validators');

var _constants = require('../constants');

/**
 * Field
 */

/* eslint-disable no-param-reassign */
var fieldNeedsValidation = exports.fieldNeedsValidation = function fieldNeedsValidation(field, name, value) {
  return field.set('needsValidation', field.get('needsValidation') || name === field.get('name') && value !== field.get('value') || field.get('syncValidators').map(function (validator) {
    return validator.get('params').some(function (param) {
      return param === name;
    });
  }).some(function (req) {
    return req;
  }));
};

var Field = exports.Field = (0, _immutable.Record)({
  name: '',
  value: '',
  syncValidators: (0, _immutable.List)(),
  syncErrors: (0, _immutable.Set)(),
  asyncValidators: (0, _immutable.List)(),
  asyncErrors: (0, _immutable.Set)(),
  serverErrors: (0, _immutable.Set)(),
  needsValidation: true,
  touched: false
});
var field = exports.field = function field() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? new Field() : arguments[0];
  var action = arguments[1];
  var type = action.type;
  var payload = action.payload;


  switch (type) {
    case _constants.ATTACH_TO_FORM:
      {
        var name = payload.name;
        var initialValue = payload.initialValue;
        var validationString = payload.validationString;
        var asyncValidators = payload.asyncValidators;


        state = state.set('name', name);
        if (initialValue) {
          state = state.set('value', initialValue);
        }
        if (validationString) {
          state = state.set('syncValidators', (0, _validators.parseSyncValidators)((0, _lodash.trim)(validationString, '|')));
        }
        if (asyncValidators) {
          state = state.set('asyncValidators', (0, _validators.parseAsyncValidators)(asyncValidators));
        }
        return state;
      }
    case _constants.BLUR:
      return state.set('touched', true);
    case _constants.TOUCH:
      // Mark as touched if no fields are specified or if it specifically
      // specified
      if (!payload.fields || payload.fields.includes(state.get('name'))) {
        return state.set('touched', true);
      }
      return state;
    case _constants.CHANGE:
      {
        state = state.set('value', payload.value);
        state = fieldNeedsValidation(state, payload.name, payload.value);
        state = state.set('asyncErrors', state.get('asyncErrors').clear());
        state = state.set('serverErrors', state.get('serverErrors').clear());
        return state;
      }
    case _constants.SUBMIT_REQUEST:
      return state.set('serverErrors', state.get('serverErrors').clear());
    case _constants.SUBMIT_FAILURE:
      return state.set('serverErrors', (0, _immutable.Set)(payload.errors[state.get('name')]));
    case _constants.VALIDATION_REQUEST:
      state = state.set('asyncValidators', state.get('asyncValidators').map(function (val) {
        return (0, _validators.asyncValidator)(val, action);
      }));
      state = state.set('asyncErrors', state.get('asyncErrors').remove(payload.validator));
      // We've requested the errors, so this doesn't need validation again until
      // the value changes.
      state = state.set('needsValidation', false);
      return state;
    case _constants.VALIDATION_NO_ERRORS:
      {
        state = state.set('asyncValidators', state.get('asyncValidators').map(function (val) {
          return (0, _validators.asyncValidator)(val, action);
        }));
        state = state.set('asyncErrors', state.get('asyncErrors').remove(payload.validator));
        return state;
      }
    case _constants.VALIDATION_ERRORS:
      {
        state = state.set('asyncValidators', state.get('asyncValidators').map(function (val) {
          return (0, _validators.asyncValidator)(val, action);
        }));
        state = state.set('asyncErrors', state.get('asyncErrors').add(payload.validator));
        return state;
      }
    case _constants.CLEAR_FORM:
      {
        if (payload && (payload.fields instanceof Array && !payload.fields.includes(state.get('name')) || typeof payload.fields === 'string' && payload.fields !== state.get('name'))) {
          return state;
        }
        state = state.set('value', '');
        state = state.set('serverErrors', state.get('serverErrors').clear());
        state = state.set('asyncErrors', state.get('asyncErrors').clear());
        state = state.set('syncErrors', state.get('syncErrors').clear());
        return state;
      }
    default:
      return state;
  }
};