'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.form = exports.Form = exports.validateForm = exports.validateField = exports.validateValue = exports.getFormData = undefined;

var _immutable = require('immutable');

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _constants = require('../constants');

var _validators = require('../validators');

var validationFunctions = _interopRequireWildcard(_validators);

var _field = require('./field');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Form
 */

var getFormData = exports.getFormData = function getFormData(form) {
  return form.get('fields').map(function (f) {
    return f.get('value');
  });
};

/* eslint-disable no-param-reassign */
var validateValue = exports.validateValue = function validateValue(value, validators, formVals) {
  var errors = [];

  validators.forEach(function (_ref) {
    var name = _ref.name;
    var params = _ref.params;

    // Confirm that the validator is recognized
    (0, _invariant2.default)(name in validationFunctions, name + ' validator is not recognized in validators.js');

    if (!validationFunctions[name](value, params, formVals)) {
      errors.push(name);
    }
  });

  return errors;
};

var validateField = exports.validateField = function validateField(f, formVals) {
  return f.set('syncErrors', (0, _immutable.Set)(validateValue(f.get('value'), f.get('syncValidators'), formVals.toJS())));
};

var validateForm = exports.validateForm = function validateForm(form) {
  return form.set('fields', form.fields.map(function (f) {
    return validateField(f, getFormData(form));
  }));
};

var Form = exports.Form = (0, _immutable.Record)({
  id: '',
  fields: (0, _immutable.Map)(),
  errors: (0, _immutable.Set)(),
  submitting: false
});
var form = exports.form = function form() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? new Form() : arguments[0];
  var action = arguments[1];
  var type = action.type;
  var payload = action.payload;


  switch (type) {
    case _constants.REGISTER_FORM:
      return state.set('id', payload);
    case _constants.ATTACH_TO_FORM:
      {
        state = state.setIn(['fields', payload.name], (0, _field.field)(undefined, action));
        state = validateForm(state);
        return state;
      }
    case _constants.DETACH_FROM_FORM:
      return state.removeIn(['fields', payload.name]);
    case _constants.CHANGE:
      {
        state = state.set('errors', state.get('errors').clear());
        state = state.set('fields', state.get('fields').map(function (f) {
          return (0, _field.fieldNeedsValidation)(f, payload.name, payload.value);
        }));
        state = state.setIn(['fields', payload.name], (0, _field.field)(state.getIn(['fields', payload.name]), action));
        state = validateForm(state);
        return state;
      }
    case _constants.SUBMIT_REQUEST:
      state = state.set('submitting', true);
      return state;
    case _constants.SUBMIT_SUCCESS:
      {
        state = state.set('submitting', false);
        state = state.set('fields', state.get('fields').map(function (f) {
          return (0, _field.field)(f, action);
        }));
        return state;
      }
    case _constants.SUBMIT_FAILURE:
      {
        state = state.set('submitting', false);
        state = state.set('fields', state.get('fields').map(function (f) {
          return (0, _field.field)(f, action);
        }));
        state = state.set('errors', (0, _immutable.Set)(payload.errors.form));
        return state;
      }
    // Change just one field specified by the `name`
    case _constants.BLUR:
    case _constants.VALIDATION_REQUEST:
    case _constants.VALIDATION_NO_ERRORS:
    case _constants.VALIDATION_ERRORS:
      {
        return state.setIn(['fields', payload.name], (0, _field.field)(state.getIn(['fields', payload.name]), action));
      }
    // Map over all the fields
    case _constants.TOUCH:
    case _constants.CLEAR_FORM:
      return state.set('fields', state.get('fields').map(function (f) {
        return (0, _field.field)(f, action);
      }));
    default:
      return state;
  }
};