'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.forms = undefined;

var _immutable = require('immutable');

var _form = require('./form');

var _constants = require('../constants');

/**
 * Forms
 */

var forms = exports.forms = function forms() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? new _immutable.Map() : arguments[0];
  var action = arguments[1];
  var type = action.type;
  var payload = action.payload;

  switch (type) {
    // Register form in reducer
    case _constants.REGISTER_FORM:
      return state.set(payload, (0, _form.form)(undefined, action));
    case _constants.UNREGISTER_FORM:
      return state.remove(payload);
    case _constants.DETACH_FROM_FORM:
      // Due to react lifecycle, the form unmounts before its children, so the
      // DETACH_FROM_FORM action triggers after UNREGISTER. Due to this, a new
      // form was created in the state with no fields. This prevents that
      // behaviour.
      return state.has(payload.id) ? state.set(payload.id, (0, _form.form)(state.get(payload.id), action)) : state;
    // For everything else, just pass down to form
    case _constants.ATTACH_TO_FORM:
    case _constants.CHANGE:
    case _constants.FOCUS:
    case _constants.BLUR:
    case _constants.TOUCH:
    case _constants.SUBMIT_REQUEST:
    case _constants.SUBMIT_SUCCESS:
    case _constants.SUBMIT_FAILURE:
    case _constants.VALIDATION_REQUEST:
    case _constants.VALIDATION_NO_ERRORS:
    case _constants.VALIDATION_ERRORS:
    case _constants.CLEAR_FORM:
      return state.set(payload.id, (0, _form.form)(state.get(payload.id), action));
    default:
      return state;
  }
};