'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseAsyncValidators = exports.asyncValidator = exports.AsyncValidator = exports.parseSyncValidators = exports.syncValidator = exports.SyncValidator = undefined;

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _immutable = require('immutable');

var _lodash = require('lodash');

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Sync validator
 */

var SyncValidator = exports.SyncValidator = (0, _immutable.Record)({
  name: '',
  params: []
});
var syncValidator = exports.syncValidator = function syncValidator() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? new SyncValidator() : arguments[0];
  var action = arguments[1];
  var type = action.type;


  switch (type) {
    default:
      return state;
  }
};

/**
 * Parse the validation string passed down from props.
 *
 * @param {string} validationString
 * @return {List<SyncValidator>}
 */
var parseSyncValidators = exports.parseSyncValidators = function parseSyncValidators(validationString) {
  // Parse validators, remove duplicates
  var strValidators = (0, _keys2.default)(validationString.split('|').reduce(function (acc, el) {
    return el ? (0, _extends4.default)({}, acc, (0, _defineProperty3.default)({}, el, null)) : acc;
  }, {}));
  return (0, _immutable.List)(strValidators).map(function (strValidator) {
    var name = strValidator;
    var params = [];

    // Parameters are separated from the validation with a colon
    if (strValidator.indexOf(':') > 0) {

      // Multiple parameters may be delimited by a comma

      var _strValidator$split = strValidator.split(':');

      var _strValidator$split2 = (0, _toArray3.default)(_strValidator$split);

      name = _strValidator$split2[0];
      params = _strValidator$split2.slice(1);
      if (params[0].indexOf(',') > -1) {
        params = params[0].split(',');
      }
    }
    // Validator names don't have to be camel case, they can be delimited by
    // dashes
    name = (0, _lodash.camelCase)(name);
    // Replace any empty string parameters with undefined, to deal with
    // params in format validate:,param
    params = params.map(function (el) {
      return (0, _lodash.isEmpty)(el) ? undefined : el;
    });

    return new SyncValidator({ name: name, params: params });
  });
};

/**
 * Async validator
 */

var AsyncValidator = exports.AsyncValidator = (0, _immutable.Record)({
  name: '',
  isValidating: false
});
var asyncValidator = exports.asyncValidator = function asyncValidator() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? new AsyncValidator() : arguments[0];
  var action = arguments[1];
  var type = action.type;
  var payload = action.payload;
  // Check if the validator matches payload validator name

  if (state.get('name') !== payload.validator) {
    return state;
  }
  switch (type) {
    case _constants.VALIDATION_REQUEST:
      return state.set('isValidating', true);
    case _constants.VALIDATION_NO_ERRORS:
    case _constants.VALIDATION_ERRORS:
      return state.set('isValidating', false);
    default:
      return state;
  }
};

/**
 * Parse the object with async validator functions.
 *
 * @param  {object} validators
 * @return {List<AsyncValidator>}
 */
var parseAsyncValidators = exports.parseAsyncValidators = function parseAsyncValidators(validators) {
  return (0, _immutable.List)((0, _keys2.default)(validators)).map(function (name) {
    return new AsyncValidator({ name: name });
  });
};