'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

exports.validate = validate;
exports.submit = submit;

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _constants = require('./constants');

var _actions = require('./actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = [validate, validateWatcher, submit, submitWatcher].map(_regenerator2.default.mark);

/**
 * VALIDATE
 */
function validate(_ref) {
  var payload = _ref.payload;
  var meta = _ref.meta;
  var successActionType, failureActionType, id, name, validator, action, responseStatus;
  return _regenerator2.default.wrap(function validate$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          successActionType = meta.successActionType;
          failureActionType = meta.failureActionType;
          id = payload.id;
          name = payload.name;
          validator = payload.validator;
          action = payload.action;

          // Dispatch the initial form request action

          _context.next = 8;
          return (0, _effects.put)(action);

        case 8:
          _context.next = 10;
          return (0, _effects.race)({
            success: (0, _effects.take)(successActionType),
            failure: (0, _effects.take)(failureActionType)
          });

        case 10:
          responseStatus = _context.sent;

          if (!responseStatus.success) {
            _context.next = 16;
            break;
          }

          _context.next = 14;
          return (0, _effects.put)((0, _actions.noAsyncErrors)({ id: id, name: name, validator: validator }));

        case 14:
          _context.next = 18;
          break;

        case 16:
          _context.next = 18;
          return (0, _effects.put)((0, _actions.receiveAsyncErrors)({ id: id, name: name, validator: validator }));

        case 18:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked[0], this);
}

function validateWatcher() {
  return _regenerator2.default.wrap(function validateWatcher$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          return _context2.delegateYield((0, _reduxSaga.takeLatest)(_constants.VALIDATION_REQUEST, validate), 't0', 1);

        case 1:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked[1], this);
}

/**
 * SUBMIT
 */
function submit(_ref2) {
  var payload = _ref2.payload;
  var meta = _ref2.meta;
  var successActionType, failureActionType, onSubmitSuccess, onSubmitFailure, id, action, responseStatus, data, errors;
  return _regenerator2.default.wrap(function submit$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          successActionType = meta.successActionType;
          failureActionType = meta.failureActionType;
          onSubmitSuccess = meta.onSubmitSuccess;
          onSubmitFailure = meta.onSubmitFailure;
          id = payload.id;
          action = payload.action;

          // Dispatch the initial form request action

          _context3.next = 8;
          return (0, _effects.put)(action);

        case 8:
          _context3.next = 10;
          return (0, _effects.race)({
            success: (0, _effects.take)(successActionType),
            failure: (0, _effects.take)(failureActionType)
          });

        case 10:
          responseStatus = _context3.sent;

          if (!responseStatus.success) {
            _context3.next = 18;
            break;
          }

          data = responseStatus.success.payload;
          _context3.next = 15;
          return (0, _effects.put)((0, _actions.submitSuccessful)({ id: id, data: data }));

        case 15:
          if (onSubmitSuccess) onSubmitSuccess();
          _context3.next = 22;
          break;

        case 18:
          errors = responseStatus.failure.payload.error.errors;
          _context3.next = 21;
          return (0, _effects.put)((0, _actions.submitFailed)({ id: id, errors: errors }));

        case 21:
          if (onSubmitFailure) onSubmitFailure();

        case 22:
        case 'end':
          return _context3.stop();
      }
    }
  }, _marked[2], this);
}

function submitWatcher() {
  return _regenerator2.default.wrap(function submitWatcher$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          return _context4.delegateYield((0, _reduxSaga.takeEvery)(_constants.SUBMIT_REQUEST, submit), 't0', 1);

        case 1:
        case 'end':
          return _context4.stop();
      }
    }
  }, _marked[3], this);
}

// All sagas to be loaded
exports.default = [submitWatcher, validateWatcher];