'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFieldAsyncErrors = exports.getFieldAsyncValidators = exports.getFieldHasSyncErrors = exports.getFieldSyncErrors = exports.getFieldSyncValidators = exports.getFieldNeedsValidation = exports.getFieldTouched = exports.getFieldValue = exports.getField = exports.getFormTouchedFields = exports.getFormFieldNames = exports.getFormIsValid = exports.getFormErrors = exports.getFormValues = exports.getFormFields = exports.getFormIsSubmitting = exports.getForm = exports.getFormsDomain = undefined;

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _reselect = require('reselect');

var _immutable = require('immutable');

var _form = require('./reducers/form');

var _field = require('./reducers/field');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Direct selector to the todos state domain
 */
var getFormsDomain = exports.getFormsDomain = function getFormsDomain() {
  return function (state) {
    return state.get('forms') || new _immutable.Map();
  };
};

/**
 * FORM SELECTORS
 */
var getForm = exports.getForm = function getForm(id) {
  return (0, _reselect.createSelector)(getFormsDomain(), function (forms) {
    return forms.get(id) || new _form.Form();
  });
};

var getFormIsSubmitting = exports.getFormIsSubmitting = function getFormIsSubmitting(id) {
  return (0, _reselect.createSelector)(getForm(id), function (form) {
    return form.get('submitting');
  });
};
var getFormFields = exports.getFormFields = function getFormFields(id) {
  return (0, _reselect.createSelector)(getForm(id), function (form) {
    return form.get('fields') || (0, _immutable.Map)();
  });
};
var getFormValues = exports.getFormValues = function getFormValues(id) {
  return (0, _reselect.createSelector)(getFormFields(id), function (fields) {
    return fields.map(function (f) {
      return f.get('value');
    });
  });
};
var getFormErrors = exports.getFormErrors = function getFormErrors(id) {
  return (0, _reselect.createSelector)(getForm(id), getFormFields(id), function (form, fields) {
    var fieldErrors = fields.map(function (f) {
      return f.get('syncErrors').merge(f.get('asyncErrors').merge(f.get('serverErrors')));
    }) || (0, _immutable.Map)();
    var allErrors = fieldErrors.set('form', form.get('errors') || (0, _set2.default)());
    return allErrors;
  });
};
var getFormIsValid = exports.getFormIsValid = function getFormIsValid(id) {
  return (0, _reselect.createSelector)(getFormErrors(id), function (errors) {
    return errors.every(function (list) {
      return list.isEmpty();
    });
  });
};
var getFormFieldNames = exports.getFormFieldNames = function getFormFieldNames(id) {
  return (0, _reselect.createSelector)(getFormFields(id), function (fields) {
    return fields.map(function (f) {
      return f.get('name');
    });
  });
};
var getFormTouchedFields = exports.getFormTouchedFields = function getFormTouchedFields(id) {
  return (0, _reselect.createSelector)(getFormFields(id), function (fields) {
    return fields.map(function (f) {
      return f.get('touched');
    });
  });
};

/**
 * Field selectors
 */
var getField = exports.getField = function getField(id, name) {
  return (0, _reselect.createSelector)(getForm(id), function (form) {
    return form.getIn(['fields', name]) || new _field.Field();
  });
};

var getFieldValue = exports.getFieldValue = function getFieldValue(id, name) {
  return (0, _reselect.createSelector)(getField(id, name), function (field) {
    return field.get('value');
  });
};
var getFieldTouched = exports.getFieldTouched = function getFieldTouched(id, name) {
  return (0, _reselect.createSelector)(getField(id, name), function (field) {
    return field.get('touched');
  });
};
var getFieldNeedsValidation = exports.getFieldNeedsValidation = function getFieldNeedsValidation(id, name) {
  return (0, _reselect.createSelector)(getField(id, name), function (field) {
    return field.get('needsValidation');
  });
};
var getFieldSyncValidators = exports.getFieldSyncValidators = function getFieldSyncValidators(id, name) {
  return (0, _reselect.createSelector)(getField(id, name), function (field) {
    return field.get('syncValidators');
  });
};
var getFieldSyncErrors = exports.getFieldSyncErrors = function getFieldSyncErrors(id, name) {
  return (0, _reselect.createSelector)(getField(id, name), function (field) {
    return field.get('syncErrors');
  });
};
var getFieldHasSyncErrors = exports.getFieldHasSyncErrors = function getFieldHasSyncErrors(id, name) {
  return (0, _reselect.createSelector)(getFieldSyncErrors(id, name), function (errors) {
    return errors.size !== 0;
  });
};
var getFieldAsyncValidators = exports.getFieldAsyncValidators = function getFieldAsyncValidators(id, name) {
  return (0, _reselect.createSelector)(getField(id, name), function (field) {
    return field.get('asyncValidators');
  });
};
var getFieldAsyncErrors = exports.getFieldAsyncErrors = function getFieldAsyncErrors(id, name) {
  return (0, _reselect.createSelector)(getField(id, name), function (field) {
    return field.get('asyncErrors');
  });
};

exports.default = getForm;